<?php

/**
 * @file
 * Admin section of the pricelist
 */


/**
 * Form callback to display form for the pricelist group
 */

function uc_pricelist_group_form($form, &$form_state, $group = NULL) {

  if (!empty($group)) {
    $pricelist_group = db_select('uc_pricelist_groups', 'ug')->fields('ug')->condition('ug.gid', $group, '=')->execute()->fetchAssoc();
  }

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Pricelist Group'),
    '#description' => t('Enter the group for the pricelist'),
    '#default_value' => isset($pricelist_group) ? $pricelist_group['title'] : '',
  );

  $form['group'] = array(
    '#type' => 'hidden',
    '#value' => $group,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Submit',
  );

  return $form;

}

/**
 * Submit callback to pricelist group
 */
function uc_pricelist_group_form_submit($form, &$form_state) {


  $values = array('title' => $form_state['values']['name']);

  if (!empty($form_state['values']['group'])) {
    //update the pricelist group
    db_update('uc_pricelist_groups')->fields($values)->condition('gid', $form_state['values']['group'], '=')->execute();
  }
  else {
    db_insert('uc_pricelist_groups')->fields($values)->execute();
  }

  //Redirect the form to prieclist list page
  $form_state['redirect'] = 'admin/store/settings/pricelists';
}

/**
 * Confirm a deletion.
 *
 * @param <type> $form_state
 * @param <type> $pricelist
 * @return <type>
 */
function uc_pricelist_group_delete_form($form, &$form_state, $group) {

  $pricelist_group = db_select('uc_pricelist_groups', 'ug')->fields('ug')->condition('ug.gid', $group, '=')->execute()->fetchAssoc();

  $form['group'] = array('#type' => 'value', '#value' => $group);
  $caption = '<p>' . t('This action cannot be undone.') . '</p>';
  return confirm_form($form, t('Are you sure you want to delete the price list %name?', array('%name' => $pricelist_group['title'])), 'admin/store/pricelists', $caption, t('Delete'), t('Cancel'));
}


/**
 * Execute pricelist group deletion
 */
function uc_pricelist_group_delete_form_submit($form, &$form_state) {

  //Deleting the pricelist group role
  db_delete('uc_pricelist_group_role')->condition('gid', $form_state['values']['group'], '=')->execute();

  //Deleting the pricelist group
  db_delete('uc_pricelist_groups')->condition('gid', $form_state['values']['group'], '=')->execute();

  //Redirect the form to prieclist list page
  $form_state['redirect'] = 'admin/store/settings/pricelists';
}

/**
 * View the pricelist group
 * @param unknown_type $form
 * @param unknown_type $form_state
 * @param unknown_type $group
 */

function uc_pricelist_group_view($group) {

  //Getting all pricelist for the current group
  $pricelists = db_select('uc_pricelist_list', 'ul')->fields('ul')->condition('ul.gid', $group, '=')->execute()->fetchAll();

  $header = array(t('PriceLists'), array('data' => t('Operations'), 'colspan' => '3'));

  $rows = array();

  $empty = '<p>You need to first ' . l(t('Create Pricelist'), 'admin/store/settings/pricelists/group/' . $group . '/add') . '</p>';

  foreach ($pricelists as $pricelist) {
    $row = array();
    $row[] = check_plain($pricelist->title);
    $row[] = array('data' => l(t('view'), 'admin/store/settings/pricelists/group/' . $pricelist->gid . '/pricelist/' . $pricelist->pid));
    $row[] = array('data' => l(t('edit'), 'admin/store/settings/pricelists/group/' . $pricelist->gid . '/pricelist/' . $pricelist->pid . '/edit'));
    $row[] = array('data' => l(t('delete'), 'admin/store/settings/pricelists/group/' . $pricelist->gid . '/pricelist/' . $pricelist->pid . '/delete'));
    $rows[] = $row;

  }

  return theme('table', array('header' => $header, 'rows' => $rows, 'empty' => $empty));
}

/*
 * The price list overview page.
 */
function uc_pricelist_overview_form() {

  // Get all of the pricelists.

  $default_pricelist = array();

  //Getting all pricelist groups
  $pricelist_groups = db_select('uc_pricelist_groups', 'ug')->fields('ug')->execute()->fetchAll();

  $default_pricelist_groups = db_select('uc_pricelist_group_role', 'gr')->fields('gr')->execute()->fetchAll();

  foreach ($default_pricelist_groups as $groups) {
    $default_pricelist[$groups->rid] = $groups->gid;
  }


  // Set the options array.
  $options = array(0 => 'None');
  if (!empty($pricelist_groups)) {
    foreach ($pricelist_groups as $name => $pricelist) {
      $options[$pricelist->gid] = $pricelist->title;
    }
  }

  // Get all of the roles.
  $roles = user_roles();
  foreach ($roles as $rid => $role) {
    $form['pricelist_roles']['list'][$rid] = array(
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => isset($default_pricelist[$rid]) ? $default_pricelist[$rid] : 0
    );
  }

  $form['pricelist_users'] = array(
    '#type' => 'item',
    '#markup' => '<p><strong>Note: You can override these settings on an individual basis by editing whatever user you would like to override, and provide a price list for that user.</strong></p>'
    );

    $form['pricelists'] = array(
    '#type' => 'fieldset',
    '#title' => t('Price Lists Group'),
    '#collapsible' => TRUE
    );

    if (count($pricelist_groups) > 0) {
      $header = array(t('Price List Group'), array('data' => t('Operations'), 'colspan' => '4'));
      $rows = array();
      foreach ($pricelist_groups as $id => $pricelist) {
        $row = array();

        $row[] = check_plain($pricelist->title);
        $row[] = array('data' => l(t('view'), 'admin/store/settings/pricelists/group/' . $pricelist->gid . '/view' ));
        $row[] = array('data' => l(t('add pricelist'), 'admin/store/settings/pricelists/group/' . $pricelist->gid . '/add'));
        $row[] = array('data' => l(t('edit'), 'admin/store/settings/pricelists/group/' . $pricelist->gid . '/edit'));
        $row[] = array('data' => l(t('delete'), 'admin/store/settings/pricelists/group/' . $pricelist->gid . '/delete'));
        $rows[] = $row;
      }
      $form['pricelists']['pricelist'] = array(
      '#type' => 'item',
      '#markup' => theme('table', array('header' => $header, 'rows' => $rows))
      );
    }
    else {
      $form['pricelists']['pricelist'] = array(
      '#type' => 'item',
      '#markup' => t('No price lists group.  You must first ' . l(t('create pricelist group'), 'admin/store/settings/pricelists/group/add') . '. '),
      );
    }

    $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Configuration'),
    '#weight' => 30,
    );

    $form['#tree'] = TRUE;

    return $form;
}

/**
 * We want to theme the overview form to make the user role selections
 * in table format.
 */
function theme_uc_pricelist_overview_form($vars) {

  $form = $vars['form'];

  $output = '';
  $header = array(t('User Role'), t('Price List Group'));
  $rows = array();
  $roles = user_roles();
  foreach (element_children($form['pricelist_roles']['list']) as $key) {
    $row = array();
    //    $rid = substr($key, 5);
    $row[] = $roles[$key];
    $row[] = drupal_render($form['pricelist_roles']['list'][$key]);
    $rows[] = $row;
  }

  $output .= theme('table', array('header' => $header, 'rows' => $rows, 'empty' => t('There is no pricelist created.')));
  $output .= drupal_render_children($form);
  return $output;
}

/**
 * Called when the overview page has been submitted.
 */

function uc_pricelist_overview_form_submit($form, &$form_state) {

  // Set the user roles
  $roles = user_roles();
  $pricelists = array();

  $pricelist_roles = $form_state['values']['pricelist_roles']['list'];

  foreach ($pricelist_roles as $rid => $gid) {
    if ($gid > 0) {

      //Checking if there is already record for the user role.
      $pricelist_query = db_select("uc_pricelist_group_role", 'gr')->fields('gr')->condition('gr.rid', $rid, '=')->execute()->fetchAssoc();

      $values = array(
        'gid' => $gid,
        'rid' => $rid
      );

      //if there is record for the user role, update it, otherwise crete new record.
      if (!empty($pricelist_query)) {
        db_update('uc_pricelist_group_role')->fields($values)->condition('id', $pricelist_query['id'], '=')->execute();
      }
      else {
        db_insert('uc_pricelist_group_role')->fields($values)->execute();
      }

      ///drupal_write_record('uc_pricelist_group_role', $values);

    }
  }

}


/**
 * The main pricelist form.
 */
function uc_pricelist_form($form, &$form_state, $group = NULL, $pid = NULL) {

  //Getting information for the pricelist
  $pricelist = db_select('uc_pricelist_list', 'up')->fields('up', array('title'))->condition('up.pid', $pid, '=')->execute()->fetchObject();

  //Getting pricelist data into csv format
  $pricelist_data = uc_pricelist_get_csv($pid);

  $help = '';
  $form['pricelist_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Price List Name'),
    '#description' => t('Enter the name for this price list'),
    '#default_value' => (!empty($pricelist) ? $pricelist->title : ''),
  );

  $form['pricelist_value'] = array(
    '#type' => 'textarea',
    '#title' => t('The Price List'),
    '#description' => t('Just copy and paste a price list, in CSV format, into this field.'),
    '#default_value' => (!empty($pricelist_data) ? $pricelist_data : ''),
  );

  $form['pid'] = array(
    '#type' => 'hidden',
    '#value' => $pid,
  );

  $form['group'] = array(
    '#type' => 'hidden',
    '#value' => $group,
  );

  $header = array('SKU', '1', '10', '20', '30', '40', '50', '60', '70', '80', '90', '100');
  $rows = array(
  array('SKU_A', '100', '90', '80', '70', '60', '50', '40', '30', '20', '10', '1'),
  array('SKU_B', '200', '190', '180', '170', '160', '150', '140', '130', '120', '110', '100'),
  array('SKU_C', '300', '290', '280', '270', '260', '250', '240', '230', '220', '210', '200'),
  array('SKU_D', '400', '390', '380', '370', '360', '350', '340', '330', '320', '310', '300'),
  );

  $help .= '<p>Let us suppose that you have 4 products (SKU_A, SKU_B, SKU_C, and SKU_D ), and for each one of those products, you would like to have different price points depending on what quantity they purchased.  ';
  $help .= 'This can be accomplished by creating a table where the first row simply illustrates the quantity ranges where the discount starts for that column.  ';
  $help .= 'Each subsequent row is then used to provide the prices for each quantity checkpoint provided within the first row.</p>';
  $help .= '<p>An example table for the following would look like the following.</p>';

  $help .= theme('table', array('header' => $header, 'rows' => $rows));

  $help .= '<p>Provided this example, if I were to purchase 55 SKU_A\'s, my charge per item would be reduced to $50, whereas I would have to pay $100 if I only bought 1.</p>';
  $help .= '<p>If you were to export this table as CSV format, you would then paste that table into the textarea above, which would look like the following.</p>';
  $help .= '<p><code>SKU,1,10,20,30,40,50,60,70,80,90,100<br/>';
  $help .= 'SKU_A,100,90,80,70,60,50,40,30,20,10,1<br/>';
  $help .= 'SKU_B,200,190,180,170,160,150,140,130,120,110,100<br/>';
  $help .= 'SKU_C,300,290,280,270,260,250,240,230,220,210,200<br/>';
  $help .= 'SKU_D,400,390,380,370,360,350,340,330,320,310,300<br/></code></p>';

  $form['help'] = array(
    '#type' => 'fieldset',
    '#title' => t('Example'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );
  $form['help']['pricelist_help'] = array(
    '#type' => 'item',
    '#markup' => $help
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Price List'),
    '#weight' => 30,
  );

  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
    '#weight' => 40,
    '#submit' => array('uc_pricelist_delete_submit'),
  );

  // $form['#validate'][] = 'uc_pricelist_validate';
  $form['#submit'][] = 'uc_pricelist_submit';

  return $form;
}

/**
 * Submit handler for the preset form.
 */
function uc_pricelist_submit($form, &$form_state) {


  $content = uc_pricelist_load_csv($form_state['values']['pricelist_value']);

  //Insert pricelist
  if (!empty($form_state['values']['pid'])) {
     $pid = $form_state['values']['pid'];
     //Update the pricelist
     db_update('uc_pricelist_list')->fields(array('title' => $form_state['values']['pricelist_name']))->condition('pid', $pid, '=')->execute();
  }
  else {
    $pid = db_insert('uc_pricelist_list')->fields(array('gid' => $form_state['values']['group'], 'title' => $form_state['values']['pricelist_name']))->execute();
  }

  $pricelist_data = $content['pricelist'];

  //If its pricelist edit, delete the existing record from the pricelist_data, so we can insert updated record into database for the pricelist.
  if (!empty($form_state['values']['pid'])) {
    db_delete('uc_pricelist_data')->condition('pid', $form_state['values']['pid'], '=')->execute();
  }

  foreach ($pricelist_data as $sku => $pricelist) {
    $prevnum = 1;
    $index = 0;
    $prevprice = 0;

    $total = count($pricelist);

    foreach ($pricelist as $num => $price) {
      if ($index > 0) {

        $values = array(
            'pid' => $pid,
            'sku' => $sku,
            'start_range' => $prevnum,
            'end_range' => ($num -1),
            'price'  => $prevprice,
        );
        //Inserting the record into database
        db_insert('uc_pricelist_data')->fields($values)->execute();
      }

      //Inserting for the last column
      if ($index == ($total - 1)) {
        $values = array(
            'pid' => $pid,
            'sku' => $sku,
            'start_range' => $num,
            'end_range' => 99999,
            'price'  => $price,
        );
        //Inserting the record into database
        db_insert('uc_pricelist_data')->fields($values)->execute();

      }

      $index++;
      $prevnum = $num;
      $prevprice = $price;

    }
  }

  $form_state['redirect'] = 'admin/store/settings/pricelists/group/' . arg(5) . '/view';

}

/**
 * Button sumit function: handle the 'Delete' button on the preset form.
 */
function uc_pricelist_delete_submit($form, &$form_state) {
  $form_state['redirect'] = 'admin/store/pricelists/manage/' . $form_state['values']['pricelist_name'] . '/delete';
}


/**
 * A table view of a pricelist.
 */
function uc_pricelist_view($group, $pid) {

  $header[] = 'SKU';
  $rows = array();
  $header_done = FALSE;

  $pricelist_data = db_select('uc_pricelist_data', 'ud')->fields('ud')->condition('ud.pid', $pid, '=')->execute()->fetchAll();

  $sku_array = array();

  //Iterating through pricelist
  foreach ($pricelist_data as $data) {

    if ($data->end_range == 99999) {
      $head_text = $data->start_range . '+';
    }
    else {
      $head_text = $data->start_range;
    }
    $row = array();

    //Adding header title
    if (!in_array($head_text, $header)) {
      $header[] = $head_text;
    }

    //Adding prices to temparory array, so we can generate rows for table
    $sku_array[$data->sku][] = $data->price;
  }

  //Generating rows for the table structure
  foreach ($sku_array as $sku => $prices) {
    $row = array();
    $row[] = array('data' => $sku);

    foreach ($prices as $price) {
      $row[] = array('data' => $price);
    }

    $rows[] = $row;
  }

  return theme('table', array('header' => $header, 'rows' => $rows, 'empty' => t('There is no pricelist')));

}

/**
 * Confirm a deletion.
 *
 * @param <type> $form_state
 * @param <type> $pricelist
 * @return <type>
 */
function uc_pricelist_delete_confirm($form, &$form_state, $pricelist) {
  $form['pricelist_name'] = array('#type' => 'value', '#value' => $pricelist['name']);
  $caption = '<p>' . t('This action cannot be undone.') . '</p>';
  return confirm_form($form, t('Are you sure you want to delete the price list %name?', array('%name' => $pricelist['name'])), 'admin/store/pricelists', $caption, t('Delete'), t('Cancel'));
}

/**
 * Execute preset deletion
 */
function uc_pricelist_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    uc_pricelist_delete($form_state['values']['pricelist_name']);
  }

  $form_state['redirect'] = 'admin/store/pricelists';
}
